==> cx_releaser-0.6.0.tar.gz: cx_releaser-0.6.0/cx_releaser/src/release.py <==
from semantic_version import Version

from cx_releaser.src.docker_registry import AwsRegistry


class Release:
    version_pattern = '[0-9]\.[0-9]\.[0-9]'

    def __init__(self, registry_client: AwsRegistry, name, version,
                 local_name=None, equal_tags=None,
                 incr_version='minor'):
        """
        :param registry_client:  registry client like AwsRegistry client
        :param name: name of image
        :param version: version (tag) of the image
        :param local_name: name of local image
        :param equal_tags: additional tags for the image (beside version)
        :param incr_version: minor, major or patch. Which version we should increment/decrement
        """
        self.registry_client = registry_client
        self.name = name
        self.equal_tags = equal_tags or []
        self.version = version
        self.local_name = local_name or f'{self.name}:{self.version}'
        self.incr_version = incr_version
        self.hash = None
        self._next = None
        self._prev = None

    def get_equal_tags_from_remote(self, hash_filter='', images=None, with_hash=False):
        equal_tags = []
        hashes = self.registry_client.get_images_on_unique_hash(self.name, hash_filter=hash_filter, images=images)
        new_hash = None
        for hash, tags in hashes.items():
            if self.version in tags:
                new_hash = hash
                equal_tags.extend(tags)
        if equal_tags:
            equal_tags.remove(self.version)
        if with_hash:
            return equal_tags, new_hash
        return equal_tags

    def exist_on_remote(self):
        hashes = self.registry_client.get_images_on_unique_hash(self.name, hash_filter='')
        for hash, tags in hashes.items():
            if self.version in tags:
                return True
        return False

    def validate_push(self, images=None, check_is_next=True, check_new_hash=False):
        images = images or self.registry_client.get_all_image_tags(self.name, tag_filter=self.version_pattern)
        if check_is_next:
            self._check_is_next_release(images)
        if check_new_hash:
            self._check_is_different_release()
        return True

    def validate_rollback(self, prev_release):
        exist = self.exist_on_remote()
        if not exist:
            raise ValueError(f'Release {self.version} does not exist')
        if prev_release and prev_release.version_sem() >= self.version_sem():
            raise ValueError('Provide one of previous releases to rollback. One with smaller version')
        if prev_release and not prev_release.exist_on_remote():
            raise ValueError(f'Release {prev_release.version} does not exist on remote. Could not rollback to it')
        return True

    def update_tags(self, new_tags=None):
        new_tags = new_tags or self.equal_tags
        self.registry_client. \
            update_image_tags(self.name, self.version, new_tags)
        return True

    def pull(self):
        return self.registry_client.pull_image(self.name, self.version)

    def push(self, validate=True, checks=None):
        checks = checks or {}
        validate and self.validate_push(check_is_next=
                                        'validate_is_next' in checks,
                                        check_new_hash=
                                        'validate_new+hash' in checks)
        result = self.registry_client.push_image(self.name, self.version, local_name=self.local_name)
        for tag in self.equal_tags:
            self.registry_client.push_image(self.name, str(tag), local_name=self.local_name)
        return result

    def rollback(self, prev_release=None,
                 validate=True, tags_to_move=None,
                 delete_equal_content_tags=False):
        validate and self.validate_rollback(prev_release)
        self.sync_with_remote()
        prev_release = prev_release or self._prev
        to_move = []
        if not delete_equal_content_tags:
            to_move = self.equal_tags[:]
        if prev_release:
            to_move.append(prev_release.version)
            if tags_to_move:
                to_move.extend([tag for tag in tags_to_move if tag not in prev_release.equal_tags])
                to_move = list(set(to_move))
                prev_release.update_tags(to_move)
        if tags_to_move:
            to_move.extend(tags_to_move)
        self.delete(with_equal_tags=True, excl_tags=to_move)

    def prev(self, unique_hash=True, prev_id=1):
        res = self.get_all_from_remote(self.name, self.registry_client,
                                       local_name=self.local_name,
                                       unique_hashes=unique_hash)
        if len(res) < 2:
            raise ValueError(f'Could not find previous release. Only {len(res)} found')
        print(f'Found {len(res)} releases')
        return res[prev_id]

    def sync_with_remote(self, images=None, version=False):
        if version:
            self.version = self.get_versions(last=True)
        self.equal_tags, self.hash = self.get_equal_tags_from_remote(with_hash=True, images=images)
        return self

    def delete(self, with_equal_tags=True, incl_tags=None, excl_tags=None):
        result = self.registry_client.delete_image(self.name, str(self.version), soft=False)
        if with_equal_tags:
            for tag in self.equal_tags:
                if incl_tags and tag not in incl_tags:
                    continue
                if excl_tags and tag in excl_tags:
                    continue
                self.registry_client.delete_image(self.name, str(tag), soft=False)
        return result

    def delete_tags(self, tags):
        for tag in tags:
            self.registry_client.delete_image(self.name, str(tag), soft=False)
        return True

    def version_sem(self):
        return Version.coerce(self.version)

    def _check_is_next_release(self, images):
        versions = []
        for image in images:
            versions.append(Version(image['tag']))
        versions.sort(reverse=True)
        if versions and versions[0] >= Version.coerce(self.version):
            raise ValueError(f'Deployed version {str(versions[0])} is not smaller than current {str(self.version)}')
        return True

    def _check_is_different_release(self):
        self.registry_client.tag_image(self.name, self.version, local_name=self.local_name)
        digset = self.registry_client.get_image_hash(local_name=self.local_name)
        result = self.registry_client.get_images_on_unique_hash(self.name,
                                                                hash_filter=digset)
        if result:
            raise ValueError(
                f'Already {len(list(result.values())[0])} tags: {list(result.values())[0]} with the same hash: {digset} has been deployed')
        return True

    def get_versions(self, last=True):
        images = self.registry_client.get_all_image_tags(self.name, tag_filter=self.version_pattern)
        versions = []
        for image in images:
            versions.append(Version.coerce(image['tag']))
        versions.sort(reverse=True)
        if last:
            return versions[0] if versions else Version.coerce('0.0.0')
        return versions

    def next(self, incr_version=None, remote_sync=False):
        if remote_sync:
            version = self.get_versions(last=True)
        else:
            version = self.version_sem()
        position = incr_version or self.incr_version
        self.version = str(getattr(version, f'next_{position}')())
        return self

    @classmethod
    def from_remote(cls, name, client: AwsRegistry,
                    local_name=None):
        images = client.get_all_image_tags(name, tag_filter=cls.version_pattern)
        if not images:
            raise ValueError(f'Could not fetch from remote. No versions found for pattern {cls.version_pattern}')
        newest = images[0]['tag']
        obj = cls(client, name, newest, local_name=local_name)
        obj.sync_with_remote()
        return obj

    @classmethod
    def get_all_from_remote(cls, name,
                            client: AwsRegistry, local_name=None,
                            unique_hashes=False):
        images_version = client.get_all_image_tags(name, tag_filter=cls.version_pattern)
        images = client.get_all_image_tags(name, tag_filter='')
        results = []
        for image in images_version:
            obj = cls(client, name, image['tag'], local_name=local_name)
            obj.sync_with_remote(images)
            results.append(obj)
        if unique_hashes:
            results = list(set(results))
        return sorted(results, reverse=True)

    @classmethod
    def from_local(cls, name, client: AwsRegistry,
                   local_name=None):
        obj = cls.from_remote(name, client, local_name=local_name)
        local_img = client.client_docker.images.get(name=local_name)
        if local_img:
            obj.local_name = local_img
        raise ValueError(f'Could not find any image {name}')

    def __hash__(self):
        return self.hash

    def __eq__(self, other):
        return self.name == other.name and self.version == other.version

    def __lt__(self, other):
        return self.name == other.name and self.version_sem() < other.version_sem()

==> cx_releaser-0.7.0.tar.gz: cx_releaser-0.7.0/cx_releaser/src/release.py <==
from semantic_version import Version

from cx_releaser.src.docker_registry import AwsRegistry


class Release:
    version_pattern = '[0-9]\.[0-9]\.[0-9]'

    def __init__(self, registry_client: AwsRegistry, name, version,
                 local_name=None, equal_tags=None,
                 incr_version='minor'):
        """
        :param registry_client:  registry client like AwsRegistry client
        :param name: name of image
        :param version: version (tag) of the image
        :param local_name: name of local image
        :param equal_tags: additional tags for the image (beside version)
        :param incr_version: minor, major or patch. Which version we should increment/decrement
        """
        self.registry_client = registry_client
        self.name = name
        self.equal_tags = equal_tags or []
        self.version = version
        self.local_name = local_name or f'{self.name}:{self.version}'
        self.incr_version = incr_version
        self.hash = None
        self._next = None
        self._prev = None

    def get_equal_tags_from_remote(self, hash_filter='', images=None, with_hash=False):
        equal_tags = []
        hashes = self.registry_client.get_images_on_unique_hash(self.name, hash_filter=hash_filter, images=images)
        new_hash = None
        for hash, tags in hashes.items():
            if self.version in tags:
                new_hash = hash
                equal_tags.extend(tags)
        if equal_tags:
            equal_tags.remove(self.version)
        if with_hash:
            return equal_tags, new_hash
        return equal_tags

    def exist_on_remote(self):
        hashes = self.registry_client.get_images_on_unique_hash(self.name, hash_filter='')
        for hash, tags in hashes.items():
            if self.version in tags:
                return True
        return False

    def validate_push(self, images=None, check_is_next=True, check_new_hash=False):
        images = images or self.registry_client.get_all_image_tags(self.name, tag_filter=self.version_pattern)
        if check_is_next:
            self._check_is_next_release(images)
        if check_new_hash:
            self._check_is_different_release()
        return True

    def validate_rollback(self, prev_release):
        exist = self.exist_on_remote()
        if not exist:
            raise ValueError(f'Release {self.version} does not exist')
        if prev_release and prev_release.version_sem() >= self.version_sem():
            raise ValueError('Provide one of previous releases to rollback. One with smaller version')
        if prev_release and not prev_release.exist_on_remote():
            raise ValueError(f'Release {prev_release.version} does not exist on remote. Could not rollback to it')
        return True

    def update_tags(self, new_tags=None):
        new_tags = new_tags or self.equal_tags
        self.registry_client. \
            update_image_tags(self.name, self.version, new_tags)
        return True

    def pull(self):
        return self.registry_client.pull_image(self.name, self.version)

    def push(self, validate=True, checks=None, test_tag=None):
        checks = checks or {}
        if test_tag:
            return self.registry_client.push_image(self.name, test_tag, local_name=self.local_name)
        validate and self.validate_push(check_is_next=
                                        'validate_is_next' in checks,
                                        check_new_hash=
                                        'validate_new+hash' in checks)
        result = self.registry_client.push_image(self.name, self.version, local_name=self.local_name)
        for tag in self.equal_tags:
            self.registry_client.push_image(self.name, str(tag), local_name=self.local_name)
        return result

    def rollback(self, prev_release=None,
                 validate=True, tags_to_move=None,
                 delete_equal_content_tags=False):
        validate and self.validate_rollback(prev_release)
        self.sync_with_remote()
        prev_release = prev_release or self._prev
        to_move = []
        if not delete_equal_content_tags:
            to_move = self.equal_tags[:]
        if prev_release:
            to_move.append(prev_release.version)
            if tags_to_move:
                to_move.extend([tag for tag in tags_to_move if tag not in prev_release.equal_tags])
                to_move = list(set(to_move))
                prev_release.update_tags(to_move)
        if tags_to_move:
            to_move.extend(tags_to_move)
        self.delete(with_equal_tags=True, excl_tags=to_move)

    def prev(self, unique_hash=True, prev_id=1):
        res = self.get_all_from_remote(self.name, self.registry_client,
                                       local_name=self.local_name,
                                       unique_hashes=unique_hash)
        if len(res) < 2:
            raise ValueError(f'Could not find previous release. Only {len(res)} found')
        print(f'Found {len(res)} releases')
        return res[prev_id]

    def sync_with_remote(self, images=None, version=False):
        if version:
            self.version = self.get_versions(last=True)
        self.equal_tags, self.hash = self.get_equal_tags_from_remote(with_hash=True, images=images)
        return self

    def delete(self, with_equal_tags=True, incl_tags=None, excl_tags=None):
        result = self.registry_client.delete_image(self.name, str(self.version), soft=False)
        if with_equal_tags:
            for tag in self.equal_tags:
                if incl_tags and tag not in incl_tags:
                    continue
                if excl_tags and tag in excl_tags:
                    continue
                self.registry_client.delete_image(self.name, str(tag), soft=False)
        return result

    def delete_tags(self, tags):
        for tag in tags:
            self.registry_client.delete_image(self.name, str(tag), soft=False)
        return True

    def version_sem(self):
        return Version.coerce(self.version)

    def _check_is_next_release(self, images):
        versions = []
        for image in images:
            versions.append(Version(image['tag']))
        versions.sort(reverse=True)
        if versions and versions[0] >= Version.coerce(self.version):
            raise ValueError(f'Deployed version {str(versions[0])} is not smaller than current {str(self.version)}')
        return True

    def _check_is_different_release(self):
        test_tag = '0.0.0-test_tag_test'
        self.push(validate=False, test_tag=test_tag)
        digset = self.registry_client.get_image_hash(local_name=self.local_name)
        result = self.registry_client.get_images_on_unique_hash(self.name,
                                                                hash_filter=digset)
        self.delete_tags([test_tag])
        if result:
            for dig, tags in result.items():
                if tags != [test_tag]:
                    raise ValueError(
                        f'Already {len(list(result.values())[0])} tags: {list(result.values())[0]} with the same hash: {digset} has been deployed')
        return True

    def get_versions(self, last=True):
        images = self.registry_client.get_all_image_tags(self.name, tag_filter=self.version_pattern)
        versions = []
        for image in images:
            versions.append(Version.coerce(image['tag']))
        versions.sort(reverse=True)
        if last:
            return versions[0] if versions else Version.coerce('0.0.0')
        return versions

    def next(self, incr_version=None, remote_sync=False):
        if remote_sync:
            version = self.get_versions(last=True)
        else:
            version = self.version_sem()
        position = incr_version or self.incr_version
        self.version = str(getattr(version, f'next_{position}')())
        return self

    @classmethod
    def from_remote(cls, name, client: AwsRegistry,
                    local_name=None):
        images = client.get_all_image_tags(name, tag_filter=cls.version_pattern)
        if not images:
            raise ValueError(f'Could not fetch from remote. No versions found for pattern {cls.version_pattern}')
        newest = images[0]['tag']
        obj = cls(client, name, newest, local_name=local_name)
        obj.sync_with_remote()
        return obj

    @classmethod
    def get_all_from_remote(cls, name,
                            client: AwsRegistry, local_name=None,
                            unique_hashes=False):
        images_version = client.get_all_image_tags(name, tag_filter=cls.version_pattern)
        images = client.get_all_image_tags(name, tag_filter='')
        results = []
        for image in images_version:
            obj = cls(client, name, image['tag'], local_name=local_name)
            obj.sync_with_remote(images)
            results.append(obj)
        if unique_hashes:
            results = list(set(results))
        return sorted(results, reverse=True)

    @classmethod
    def from_local(cls, name, client: AwsRegistry,
                   local_name=None):
        obj = cls.from_remote(name, client, local_name=local_name)
        local_img = client.client_docker.images.get(name=local_name)
        if local_img:
            obj.local_name = local_img
        raise ValueError(f'Could not find any image {name}')

    def __hash__(self):
        return self.hash

    def __eq__(self, other):
        return self.name == other.name and self.version == other.version

    def __lt__(self, other):
        return self.name == other.name and self.version_sem() < other.version_sem()
